/**
 * @file Parent.c
 * @author Hugo Tavares nº 27764 (hugo.tavares@ua.pt)
 * @brief Ficheiro que contém o processo Pai
 * 
 * Este ficheiro conteém o processo Pai que trata da comunicação série.
 * Isto é, envia e recebe comandos pela porta série (XBee)  para os SALbot.
 */

#define _PARENT_

#include "SALbot_Interface.h"

/**
 * @fn int parentMain(int pid, str_msg* pf_mem, int pc)
 * Função main do Pai.\n
 * O Pai está encarregue de enviar e receber comandos pela porta série RS232.
 * Quando recebe algum comando escreve o que recebeu na \c shared \c memory e envia
 * um "sinal" ao Filho a dizer que já pode ir ver o que recebeu.\n
 * Por outro lado o Pai sempre que receba um "sinal" do Filho, envia os comandos
 * que estiverem na \c shared \c memory.
 * @param pid pid do filho.
 * @param pf_mem shared memory (estrutura do tipo str_msg).
 * @param pc inteiro que representa o tipo de porta a usar para a comunicação.
 */
int parentMain(int pid, str_msg* pf_mem, int pc)
{
	unsigned char *buf, *str, msg[20];
	int n, fd, nr, nw;
	unsigned char type, msg_check, len, byte_test;
	struct termios oldtio, newtio;
	
	/**
	 * Aloca espaço de memória para o array de caracteres(string) @a str .
	 */
	str = malloc(100*sizeof(char));
	buf = str;
	
	/**
	 * @a fd Abre a porta de comunicações série como sendo de leitura e escrita,
	 * sem estar à espera.Consoante o que o utilizador escolheu ao carregar o programa @a argv[1] .
	 */
	switch(pc)
	{
		case 1:
			fd = open("/dev/ttyS0",O_RDWR | O_NOCTTY | O_NDELAY /*| O_NONBLOCK*/);
			break;
		case 2:
		       fd = open("/dev/ttyUSB0",O_RDWR | O_NDELAY | O_NOCTTY /*| O_NONBLOCK*/);
			break;
		case 3:
			fd = open("/dev/ttyUSB1",O_RDWR | O_NOCTTY | O_NDELAY /*| O_NONBLOCK*/);
			break;
	}
	if(fd == -1)
	{
		printf("\n");
		textcolor(RESET,WHITE,RED);
		printf(" Erro ");
		textcolor(RESET,RED,-30);
		printf(" Não foi possivel abrir a porta série!\n");
		resetcolor();
		return 1;
	}
	tcgetattr(fd, &oldtio); /*guarda os valores antigos*/
	bzero(&newtio, sizeof(newtio));
	
	newtio.c_cflag = B19200 | CS8 | CREAD | CLOCAL;
	newtio.c_cflag &= ~HUPCL;	////////////////////////////////////////////////////////
	newtio.c_lflag = 0;	/////////////////////////////////////////////
	newtio.c_iflag = ICRNL | IGNPAR;
	newtio.c_iflag = 0;	/////////////////////////////////////////
	newtio.c_cc[VTIME] = 0;	//////////////////////////////////////////////
	newtio.c_cc[VMIN] = 0;	//////////////////////////////////////////
	newtio.c_iflag = IGNBRK | IGNPAR;
	
	tcflush(fd, TCIFLUSH);
	tcsetattr(fd, TCSANOW, &newtio);
	
	fcntl(fd, F_SETFL, FNDELAY); ///////////////////////////////////////

	/**
	 * Conecta os sinais \c SIGUSR1 e \c SIGINT para o Pai os poder intreceptar.
	 */
	signal(SIGUSR1, parentSignalHandler);
	signal(SIGINT, parentKillHandler);

	while(pf_msgG->Com != Exit)
	{
		/**
		 * Vai lendo da porta série. Em modo não bloqueante.
		 */
		//if(pf_msgG->Com == Receber)
		while(pf_msgG->Com == Receber)
		//while(1)
		{
			/**
			* @a nr Lê o que está sendo recebido na porta série, caracter a caracter
			*/
			pausa(100);
			nr = read(fd,buf,1);
			
			/**
			* Quando encontrar o caracter 0xFF (inicio de mensagem)
			* vai processar a mensagem, isto é vai procurar qual o tipo de mensagem recebido
			*/
			if(((unsigned char)*buf == (unsigned char)0xFF) || ((unsigned char)*buf == (unsigned char)0xFE))
			{
			       switch(*buf)
			       {
				      case 0xFF:
					     pf_msgG->Robot = SALbot1;
					     break;
				      case 0xFE:
					     pf_msgG->Robot = SALbot2;
					     break;
				      default:
					     break;
			       }
			       buf++;
			       nr = read(fd,buf,1);
			       type = (unsigned char)*buf<<4;
			       type = type>>4;
			       switch(type)
			       {
				      case 4:
					     pf_msgG->Msg.tipo = Estado_SAL;
					     break;
				      case 7:
					     pf_msgG->Msg.tipo = Q_table;
					     break;
				      case 8:
					     pf_msgG->Msg.tipo = Learn_progress;
					     break;
				      case 9:
					     pf_msgG->Msg.tipo = Action_executed;
					     break;
				      case 10:
					     pf_msgG->Msg.tipo = Sensors_dist;
					     break;
				      case 12:
					     pf_msgG->Msg.tipo = Sensors_bumper;
					     break;
				      default:
					     pf_msgG->Msg.tipo = none;
					     break;
			       }
			       if(pf_msgG->Msg.tipo != none)
			       {
				      msg_check = (unsigned char)*buf;
				      len = (unsigned char)*buf>>4;
				      for(n=0;n<len;n++)
				      {
					     buf++;
					     nr = read(fd,buf,1);
					     msg_check ^= (unsigned char)*buf;
				      }
				      buf++;
				      nr = read(fd,buf,1);
				      byte_test = (unsigned char)*buf;
				      if(byte_test == msg_check)
				      {
					     unsigned int d; //
					     fflush(stdout); /**< Força os printf's*/
					     printf("\nstring recebida(hexadécimal)"); //
					     for(d=0;d<len+3;d++) //
					     {
					     	printf(" %X", str[d]);
					     }
					     printf("\n"); //
					     printf("\nstring recebida(décimal)");  //
					     for(d=0;d<len+3;d++) //
					     {
						    printf(" %d", str[d]);
					     }
					     printf("\n"); //
					     
					     *buf = 0; //- ^
					     printf("\n"); //- ^
					     /**
					     * Função que processa os dados recebidos
					     */					     
					     sfc(pid,str);
					     
					     printf("\n");
				      }
			       }				
				/**
				 * Reinicia a posição do primeiro ponteiro buf para o endereço do primeiro
				 * byte de str
				 */
				str[0] = 0;
				buf = &str[0];
				printline();
			}
		}
		
		/**
		 * Vai escrever na porta série.
		 */
		if(pf_msgG->Com == Enviar)
		{
		       unsigned int g;
		       
		       Processa_msg();
		       for(g=0;g<pf_msgG->send_msg.len;g++)
		       {
			      msg[g] = pf_msgG->send_msg.msg[g];
		       }
		       
		       printf("\nstring enviada(hexadécimal)");
		       for(g=0;g<pf_msgG->send_msg.len;g++)
		       {
			      printf(" %X",msg[g]);
		       }
		       printf("\n");
		       printf("\n");
		       printf("\nstring enviada(décimal)");
		       for(g=0;g<pf_msgG->send_msg.len;g++)
		       {
			      printf(" %d",msg[g]);
		       }
		       printf("\n");
		       nw =write(fd, msg, pf_msgG->send_msg.len);
			pausa(100);
			if (nw<0)
			{
				textcolor(RESET,BLUE,-30);
				printf("Pai: ");
				textcolor(RESET,WHITE,BLUE);
				printf("Erro!!! Não foi possivel Enviar a mensagem pela porta série!");
				resetcolor();
				printf("\n");
				printline();
				pausa(3000);
			}
			else
			{
			       int l;
			       
			       for(l=0;l<20;l++)
			       {
				      pf_msgG->send_msg.msg[l] = 0;
			       }
			       textcolor(RESET,BLUE,-30);
			       printf("Pai: ");
			       textcolor(RESET,WHITE,BLUE);
			       switch(pf_msgG->send_msg.Cmd)
			       {
				      case 1:
					     printf("Enviei o comando para alteração do modo operacional para Autonomo");
					     break;
				      case 2:
					     printf("Enviei o comando para alteração do modo operacional para Manual");
					     break;
				      case 3:
					     printf("Enviei o comando para iniciação da Aprendizagem");
					     break;
				      case 4:
					     printf("Enviei o comando para Aprendizagem");
					     break;
				      case 5:
					     printf("Enviei o comando para requsição de dados da tabela de Aprendizagem");
					     break;
				      case 6:
					     printf("Enviei o comando para requsição de dados dos sensores de distância");
					     break;
				      case 7:
					     printf("Enviei o comando para requsição do estado do sistema de para-choques");
					     break;
				      case 8:
					     if(pf_msgG->send_msg.Tipo == req_cmd)
					     {
						    printf("Enviei o comando para apagar a tabela de aprendizagem no SALbot");
					     }
					     break;
				      default:
					     break;
			       }
			       resetcolor();
			       printf("\n");
			       printline();
			       pf_msgG->request_to_send = TRUE;
			       pf_msgG->Com = Receber;
			}
		}
	}
	
	/**
	 * "Desatacha" a \c shared \c memory da estrutura de dados pf_mem
	 */
	shmdt((void*)pf_mem);
	close(fd);
	g_print("Tambem vou Sair!!!\n");
	return 0;
}

/**
 * @fn void parentSignalHandler(int dumm)
 * Interrupt activado pelo sinal \c SIGUSR1
 * Altera o estado de comunicação para Enviar.
 */
void parentSignalHandler(int dumm)
{
	pf_msgG->Com = Enviar;
	return;
}

/**
 * @fn parentKillHandler(int dumm)
 * Interrupt activado pelo sinal \c SIGINT (ctrl +c).
 * Intercepta o sinal para que o programa não termine.
 */
void parentKillHandler(int dumm)
{
	textcolor(RESET,BLUE,-30);
	printf("Pai: ");
	textcolor(RESET,WHITE,BLUE);
	printf("Não quero morrer já!!");
	resetcolor();
	printf("\n");
	printline();
	return;
}
