/**
* @file UART.c
* @author Hugo Tavares nº 27764 (hugo.tavares@ua.pt)
* @brief Contem funções que permitem o envio e recepção de dados pela porta série.
*
* 
*/

/*********************************************************************
 *
 *                  UART Routines
 *
 *********************************************************************
 * FileName:        UART.c
 * Dependencies:
 * Processor:       PIC18
 * Complier:        HI-TECH PICC18
 ********************************************************************/

#include "UART.h"
#include <htc.h>


#if defined(__PICC18__)

/* Se não estiverem definidos os valores da frequência de clock nem 
 * do Baudrate então coloca os valores por defeito. */

#ifndef CLOCK_FREQ
	#define CLOCK_FREQ	4000000
#endif

#ifndef	BAUD_RATE
	#define	BAUD_RATE	19200
#endif

//#define USART_USE_BRGH_HIGH
#define USART_USE_BRGH_LOW

/*#if defined(USART_USE_BRGH_LOW)
    #define SPBRG_VAL   ( ((CLOCK_FREQ/BAUD_RATE)/16) - 1)	
#else
   	#define SPBRG_VAL   ( ((CLOCK_FREQ/BAUD_RATE)/64) - 1)
#endif*/

#define SPBRG_VAL	12


#if SPBRG_VAL > 255
    #error "Calculated SPBRG value is out of range for currnet CLOCK_FREQ."
#endif

/**
 * @fn UARTInit()
 * Inicialização da UART.
 */
void UARTInit(void)
{
    SPEN = 1;   // (RCSTA<7>) must be set (= 1),
    TRISC6 = 0;
    TRISC7 = 1;

#if defined(USART_USE_BRGH_HIGH)
    TXSTA = 0x24;
#else
    TXSTA = 0x20;
#endif

    RCSTA = 0x90; // 0b10010000;
    SPBRG = SPBRG_VAL;
    BAUDCON = 0x40;
}

/**
* @fn UARTPut_String(unsigned char* str)
* Função que envia uma string pela porta série.
* @param str A string a enviar
*/
void UARTPut_String(unsigned char* str)
{
    BYTE c;
	
	TXREG = 0;
	
    while( c = *str++ )
        UARTPut((unsigned char)c);
        
	while(!TXIF)continue;
	PIR1bits.TXIF=0;
}

/**
* @fn BYTE UARTGetString(char *buffer, BYTE bufferLen)
* Função que lê uma string da porta série .
* @param buffer Buffer que contem a string
* @param bufferLen Comprimento da string
* @return Devolve o nº de ciclos realizados
*/
BYTE UARTGetString(char *buffer, BYTE bufferLen)
{
    BYTE v;
    BYTE count;

    count = 0;
    do
    {
        if ( bufferLen-- == 0 )
            break;

        while( !UARTIsGetReady() );

        v = RCREG;

        if ( v == '\r' || v == '\n' )
            break;

        count++;
        *buffer++ = v;
        *buffer = '\0';
    } while(1);
    return count;
}

/**
* @fn UARTPut(BYTE c)
* Função que envia um byte pela porta série .
* @param c Byte a enviar
*/
void UARTPut(BYTE c)
{
    while( !UARTIsPutReady() );
    TXREG = c;
}

/**
* @fn UARTPutString(BYTE *s)
* Função que envia uma string da porta série .
* @param c String a enviar
*/
void UARTPutString(BYTE *s)
{
    BYTE c;

    while( (c = *s++) )
        UARTPut(c);
    while( !UARTIsPutReady() );
}

/**
* @fn BYTE UARTGet(void)
* Função que lê um byte da porta série .
* @return Devolve o byte lido
*/
BYTE UARTGet(void)
{
    // Clear overrun error if it has occured
    // New bytes cannot be received if the error occurs and isn't cleared
    if(OERR)
    {
        CREN = 0;   // Disable UART receiver
        CREN = 1;   // Enable UART receiver
        printf("\r\nOcorreu um erro de overrun\r\n");
    }

    return RCREG;
}

#if 1
unsigned unsigned char CharacterArray[]={'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'};
/**
* @fn PrintChar(BYTE toPrint)
* Função que envia um byte pela porta série no fomato hexadécimal.
* @param toPrint byte para enviar
*/
void PrintChar(BYTE toPrint)
{
    BYTE PRINT_VAR;
    PRINT_VAR = toPrint;
    toPrint = (toPrint>>4)&0x0F;
    UARTPut(CharacterArray[toPrint]);
    toPrint = (PRINT_VAR)&0x0F;
    UARTPut(CharacterArray[toPrint]);
    return;
}
#endif

#else
	#error Unknown processor.  See Compiler.h
#endif
