/**
* @file source.c
* @author Hugo Tavares nº 27764 (hugo.tavares@ua.pt)
* @brief Rotina principal do PIC18F4620
*
* 
*/

/*-*******************************************************************
 *
 *                  Main Routine
 *
 *********************************************************************
 * FileName:        source.c
 * Dependencies:
 * Processor:       PIC18
 * Complier:        HI-TECH PICC18
 ********************************************************************/
#include <htc.h>
#include "UART.h"
#include <Delay.c>
#include <itoa.c>

/*-*******************************************************************
 * 
 *	Configuração do Processador
 * 
 ********************************************************************/
/**
 * @name Configuração do PIC
* Configuração do microprocessador
 */
/*@{*/
__CONFIG(1, HSPLL);
__CONFIG(2, BOREN & BORV21 & PWRTDIS & WDTPS512 & WDTDIS);
__CONFIG(3, MCLREN & PBDIGITAL);
__CONFIG(4, DEBUGDIS & STVRDIS);
/*@}*/

/*-*******************************************************************
 * 
 *	Macros
 * 
 ********************************************************************/
/**
 * @name Macros
*  Macros que permitem manipular variáveis de 16 bits
*/
/*@{*/
#define	high_byte(x)				(unsigned char) (x>>8)
#define	low_byte(x)				(unsigned char)	(x & 0xFF)
#define	make8(var,offset)			((unsigned int)var >> (offset * 8)) & 0x00FF
#define	make16(varhigh,varlow)	(((unsigned int)varhigh & 0xFF)* 0x100) + ((unsigned int)varlow & 0x00FF)
/*@}*/

/*-*******************************************************************
 * 
 *	Configuração de variáveis    
 * 
 ********************************************************************/
/**
 * @name Defines de variáveis
*  Atribuição de variáveis a saídas do microcontrolador
*/
/*@{*/
#define		LED1	LATB1
#define		LED2	LATB2
#define		LED3	LATB3
#define		LED4	LATB4
#define		LED6	LATD6
#define		LED7	LATD7
#define		pwm	LATA1
#define		gp2d12	LATA0
#define		lv_ez0	LATA1
/*@}*/

/*-*******************************************************************
 * 
 *	Defines
 * 
 ********************************************************************/
/**
* @name Defines
*  Defines
*/
/*@{*/
#define	ADNS	1
#define	GP2D12	2
#define	LV_EZ0	3

#define	ADNS_len	4
#define GP2D12_len	2
#define LV_EZ0_len	2

#define	size_of_buff	30
/*@}*/

/*-*******************************************************************
 * 
 *	Enums
 * 
 ********************************************************************/
/**
* @enum Bool_enum
* Booleano ON-OFF.
*/
typedef enum
{
	OFF = 0,
	ON = 2	
} Bool_enum;	

/**
* @enum Modo_op_enum
* Modos de operação possíveis do programa do microcontrolador.
*/
typedef enum
{
	DEBUG,
	Autonomo,
	Manual,
	Idle
} Modo_op_enum;

/**
* @enum msg_enum
* Tipos de mensagens que se pode receber.
*/
typedef enum
{
	none,
	op_mode_cmd=4,
	req_cmd,
	start_stop_cmd=15
} msg_enum;

/**
* @enum cmd_enum
* Mensagens que se podem receber.
*/
typedef enum
{
	autonomo=1,
	manual,
	mouse,
	gp2d120,
	lvez0,
	iniciar=85,
	parar=170
} cmd_enum;	

/*-*******************************************************************
 * 
 *	Estruturas
 * 
 ********************************************************************/
/**
* @struct Man_send_flags
* Flags que representam o tipo de mensagem que o microcontrolador
* tem de enviar, quando está no modo Manual.
*/
typedef struct
{
	Bool_enum	mouse;
	Bool_enum	gp2d12;
	Bool_enum	lv_ez0;
} Man_send_flags;

/**
* @struct Mouse_struct
* Estrutura de dados que contém os dados obtidos do dispositivo 
* ADNS-2610(Rato).
* @note Os campos \c x_char e \c y_char apenas servem para debug.
*/
typedef struct
{
	unsigned char	x_sign;
	unsigned char	dx;
	unsigned char	x_char[3];
	unsigned char	y_sign;
	unsigned char	dy;
	unsigned char	y_char[3];
} Mouse_struct;

/**
* @struct Dist_struct
* Estrutura de dados que serve para armazenar os dados dos
* sensores de distância \c GP2D120 e \c LV-EZ0.
*/
typedef struct
{
	unsigned char	valH;
	unsigned char	valL;
} Dist_struct;

/**
* @struct Estrutura_geral
* Estrutura de dados que contém os valores dos sensores de distância,
* os dados do rato, as flags de tipo de dados a enviar(modo Manual)
* e o modo de operação do programa do microcontrolador.
*/
typedef struct
{
	Modo_op_enum	Modo_op;
	Mouse_struct		Mouse;
	Dist_struct		Sonar;
	Dist_struct		IFR;
	msg_enum		Received_type;
	cmd_enum		Received_cmd;
	Man_send_flags	Send;
} Estrutura_geral;

/*-*******************************************************************
 * 
 *	Protótipos
 * 
 ********************************************************************/

void init(void);
void Execute_Cmd(unsigned char *);
void Process_Commands(void);
void Get_dist(void);
int is_get_ready(void);
unsigned char UART_Get(void);
void mouse_sens_writeByte(unsigned int);
signed char mouse_sens_readByte(void);
void mouse_sens_write(unsigned int adr, unsigned int);
signed char mouse_sens_read(unsigned int);
void Send_Command(int);

/*********************************************************************
 * 
 *	Variáveis
 * 
 ********************************************************************/

Estrutura_geral	EstadoG;

signed char x_mov, y_mov;

unsigned int i, t=0, wre, buff, count = 10, junk;
unsigned char *actualbuf, *buffer, *lastbuf, *buf;
unsigned char  msg[size_of_buff], str[20];
int j;


/*-*******************************************************************
 * 
 *	Main
 * 
 ********************************************************************/
/**
* @fn main()
* Rotina principal do programa do microcontrolador.\n
* Basicamente faz a inicialização do microcontrolador e inicia um ciclo infinito, no qual faz
* uma aquisição dos valores do incremento de x e y do Rato, actualiza o estado dos LED's
* (indicando a movimentação do Rato), depois envia uma mensagem com esses valores
* pela porta série. Em seguida faz a aquisição dos sensores de distância, e envia esses valores
* pela porta série. Por fim inicia uma rotina que vai verificar se recebeu algum tipo de mensagem
* pela porta série.\n
* No final de cada ciclo, foi necessário colocar um \c Delay de 875 ms que na verdade são,
* apróximadamente, 219 ms, pois o pic funciona no modo PLL (  \f$ F_{osc,pic} = 4 \times F_{osc,xtal} \f$ )
* @note o programa começa em modo Idle, neste caso a rotina apenas executa a função que
* vai verificar se existe alguma mensagem na porta série.
*/
void main()
{		
	buffer = &msg[0];
	actualbuf = &msg[0];
	lastbuf = &msg[size_of_buff-1];
	buf = &str[0];
	
	init();
	
	mouse_sens_write(0x00, 0x80);	//Faz reset ao sensor
	
	mouse_sens_write(0x00, 0x01);	//Mantem o sensor "acordado"

	LED1 = 1;
	DelayMs(500);
	LED1 = 0;
	
	EstadoG.Modo_op = Idle;
	//UARTPut_String( (unsigned char *)"Ola Bem Vindo!\r\n");

	while(1)
	{
		if(EstadoG.Modo_op != Idle)
		{
			y_mov = (mouse_sens_read(0x02));
			x_mov = (mouse_sens_read(0x03));
					
			if(x_mov < 0)
			{
				LED1 = 1;
				LED3 = 0;
			}
			else if(x_mov > 0)
			{
				LED3 = 1;
				LED1 = 0;
			}
			else
			{
				LED1 = 0;
				LED3 = 0;
			}
			
			if(y_mov < 0)
			{
				LED2 = 1;
				LED4 = 0;
			}
			else if(y_mov > 0)
			{
				LED4 = 1;
				LED2 = 0;
			}
			else
			{
				LED2 = 0;
				LED4 = 0;
			}
			
			if(x_mov < 0)
			{
				EstadoG.Mouse.x_sign = '-';
				EstadoG.Mouse.dx = ~x_mov + 1;
			}	
			else
			{
				EstadoG.Mouse.x_sign = ' ';
				EstadoG.Mouse.dx = x_mov;
			}
			if(y_mov < 0)
			{
				EstadoG.Mouse.y_sign = '-';
				EstadoG.Mouse.dy = ~y_mov + 1;
			}		
			else
			{
				EstadoG.Mouse.y_sign = ' ';
				EstadoG.Mouse.dy = y_mov;
			}
			
			if(EstadoG.Modo_op != Manual || EstadoG.Send.mouse == ON)
			{
				/* Limpa a flag de envio no modo manual */
				if(EstadoG.Send.mouse == ON)
					EstadoG.Send.mouse = OFF;
					
				Send_Command(ADNS);
			}
			
			Get_dist();
			
			if(EstadoG.Modo_op != Manual || EstadoG.Send.gp2d12 == ON)
			{
				/* Limpa a flag de envio no modo manual */
				if(EstadoG.Send.gp2d12 == ON)
					EstadoG.Send.gp2d12 = OFF;
					
				Send_Command(GP2D12);
			}
			
			if(EstadoG.Modo_op != Manual || EstadoG.Send.lv_ez0 == ON)
			{
				/* Limpa a flag de envio no modo manual */
				if(EstadoG.Send.lv_ez0 == ON)
					EstadoG.Send.lv_ez0 = OFF;
					
				Send_Command(LV_EZ0);
			}
		}		
		
		if(is_get_ready())
		{
			Process_Commands();
		}
		
		/*DelayMs(500);
		DelayMs(500);
		DelayMs(500);
		DelayMs(500);
		DelayMs(500);
		DelayMs(500);*/
		DelayMs(125);
		DelayMs(250);
		DelayMs(250);
		DelayMs(250);
	}		
}

/*-*******************************************************************
 * 
 *	Iniciação
 * 
 ********************************************************************/
/**
 * @fn init(void)
 * Inicializa as portas do PIC bem como configura os bancos
 * necessários para a utilização do conversor analógico, UART
 * e interrupts.
 */
void init(void)
{	
	ADIE	= 0;	// Desliga o interrupt do conversor A/D
	ADCON1	= 0b00001101;
	//ADCON2	= 0b10000110;
	ADCON2	= 0b10111111;
	ADON	= 1;	// Liga o conversor A/D
	
	PORTA	= 0;
	TRISA	= 0b00111111;
	PORTB	= 0;
	
	TRISB	= 0b00000000;
	PORTD	= 0;
	TRISD7	= 0;
	TRISD6	= 0;
		/* Configuração da comunicação por rs232 */
		
	UARTInit();
	
	TRISC5  = 0;
	TRISC3  = 0;
	
	RCIE = 1;
	PEIE = 1;
	GIE = 1;
	
}	

/*-*******************************************************************
 * 
 *	Função de envio de dados para o rato
 * 
 ********************************************************************/
 
/**
 * @fn mouse_sens_writeByte(unsigned int dados)
 * Transmite um Byte para o sensor ADNS-2610.
 * @param dados o Byte transmitido
 */
void mouse_sens_writeByte(unsigned int dados)
{
	unsigned int i;
    TRISC5 = 0; 		// SDA como Output
	
	for (i=0; i<8; i++){
		RC3 = 0;		// SCK =0
		
		//Bit a enviar
		if(dados & 0x80)
		{
			RC5 = 1;			
		}
		else
		{
			RC5 = 0;
		}
		dados = dados <<1;		// Shifta o Byte de dados para preparar o bit a enviar
		
		DelayUs(1);
		
		RC3 = 1;		// SCK =1
		
		DelayUs(1);
	}
	
    TRISC5 = 1; 		// SDA como Input
}

/*-*******************************************************************
 * 
 *	Função de recepção de dados do rato
 * 
 ********************************************************************/
 
/**
 * @fn char mouse_sens_readByte()
 * Lê um Byte do sensor ADNS-2610.
 * @return Devolve o Byte lido.
 */
signed char mouse_sens_readByte(void)
{
	int i;
	int dados=0;

    TRISC5 = 1; 		// SDA como Input

	for (i=0; i<8; i++){
		RC3 = 0;		// SCK =0 
		dados=dados<<1;					// Shifta o Byte de dados para receber o próximo bit

		DelayUs(1);
		RC3 = 1;		// SCK =1
		
		dados |= RC5;			//Dados lidos
		
		DelayUs(1);
	}

	return dados;
}

/*-*******************************************************************
 * 
 *	Função de escrita de dados no rato
 * 
 ********************************************************************/

/**
 * @fn mouse_sens_write(unsigned int adr, unsigned int dados)
 * Escreve dados num endereço do sensor ADNS-2610.
 * @param adr o Endereço
 * @param dados os Dados
 */
void mouse_sens_write(unsigned int adr, unsigned int dados)
{

	mouse_sens_writeByte(adr|=0x80);  //o primeiro bit a enviar deve ser 1 (Operação de escrita)
	mouse_sens_writeByte(dados);
	DelayMs(100);	//pelo menos 100 micro-segundos de espera
}

/**
 * @fn char mouse_sens_read(unsigned int adr)
 * Envia um endereço ao sensor ADNS-2610 para ler o seu conteudo
 * e lê o Byte desse endereço.
 * @param adr o Endereço
 * @return os Dados
 */
signed char mouse_sens_read(unsigned int adr)
{
	mouse_sens_writeByte(adr);
	return mouse_sens_readByte();
}

/**
 * @fn Get_dist()
 * Função que adquire os valores dos sensores de distância \c GP2D120
 * e \c LV-EZ0 através da conversão A/D.
 */
void Get_dist(void)
{
	/* Leitura do sensor GP2D12 */
	CHS3 = 0;
	CHS2 = 0;
	CHS1 = 0;
	CHS0 = 0;
	
	GODONE = 1;
	while(GODONE){}
	ADIF = 0;
	
	EstadoG.IFR.valH = ADRESH;
	EstadoG.IFR.valL = ADRESL;
	
	/* Leitura do sensor LV EZ0 */
	CHS3 = 0;
	CHS2 = 0;
	CHS1 = 0;
	CHS0 = 1;
	
	GODONE = 1;
	while(GODONE){}
	ADIF = 0;
	
	EstadoG.Sonar.valH = ADRESH;
	EstadoG.Sonar.valL = ADRESL;
}	

/**
 * @fn Send_Command(int tipo)
 * Função que envia uma mensagem pela porta série.
 * @param tipo Tipo de mensagem a enviar
 * @note Se o modo for de Debug não envia uma mensagem, mas sim
 * um conjunto de caracteres e dados que permitem ao utilizador
 * visualizar num terminal os dados que o PIC esta a adquirir dos sensores.
 */
void Send_Command(int tipo)
{
	unsigned char Tipo;
	unsigned char byte_check;
	switch(tipo)
	{
		case ADNS:
			Tipo = ADNS_len;
			Tipo = Tipo << 4;
			Tipo += ADNS;
			
			if(EstadoG.Modo_op == DEBUG)
			{
				int base=10;
				unsigned int i,n;
				
				/* Escreve os dados de X */
				printf( (unsigned char *)"\r\nX: ");
				PrintChar(EstadoG.Mouse.dx);
				
				utoa(EstadoG.Mouse.x_char,EstadoG.Mouse.dx,base);
				
				printf( (unsigned char *)" , itoa: ");
				
				if(EstadoG.Mouse.dx > 99)
				{
					n = 3;
				}
				else if(EstadoG.Mouse.dx > 9)
				{
					n = 2;
					UARTPut(' ');
					EstadoG.Mouse.x_char[2] = 0;
				}
				else
				{
					n = 1;
					UARTPut(' ');
					UARTPut(' ');
					EstadoG.Mouse.x_char[2] = 0;
					EstadoG.Mouse.x_char[1] = 0;
				}
				UARTPut(EstadoG.Mouse.x_sign);
				
				for(i=0;i<n;i++)
				{
					UARTPut(EstadoG.Mouse.x_char[i]);
					EstadoG.Mouse.x_char[i] = 0;
				}
				
				/* Escreve os dados de Y */
				printf( (unsigned char *)"; Y: ");
				PrintChar(EstadoG.Mouse.dy);
				
				utoa(EstadoG.Mouse.y_char,EstadoG.Mouse.dy,base);
				
				printf( (unsigned char *)" , itoa: ");
				
				if(EstadoG.Mouse.dy > 99)
				{
					n = 3;
				}
				else if(EstadoG.Mouse.dy > 9)
				{
					n = 2;
					UARTPut(' ');
					EstadoG.Mouse.y_char[2] = 0;
				}
				else
				{
					n = 1;
					UARTPut(' ');
					UARTPut(' ');
					EstadoG.Mouse.y_char[2] = 0;
					EstadoG.Mouse.y_char[1] = 0;
				}
				UARTPut(EstadoG.Mouse.y_sign);
				
				for(i=0;i<n;i++)
				{
					UARTPut(EstadoG.Mouse.y_char[i]);
					EstadoG.Mouse.y_char[i] = 0;
				}
				
				printf( (unsigned char *)" ;\r\n");
			}
			else
			{
				byte_check = Tipo;
				byte_check ^= EstadoG.Mouse.x_sign;
				byte_check ^= EstadoG.Mouse.dx;
				byte_check ^= EstadoG.Mouse.y_sign;
				byte_check ^= EstadoG.Mouse.dy;
				UARTPut(255);
				UARTPut(Tipo);
				UARTPut(EstadoG.Mouse.x_sign);
				UARTPut(EstadoG.Mouse.dx);
				UARTPut(EstadoG.Mouse.y_sign);
				UARTPut(EstadoG.Mouse.dy);
				UARTPut(byte_check);
			}
			break;
		case GP2D12:
			Tipo = GP2D12_len;
			Tipo = Tipo << 4;
			Tipo += GP2D12;
			
			if(EstadoG.Modo_op == DEBUG)
			{
				unsigned int valor, i, n;
				unsigned char valor_c[4];
				int base=10;
				
				valor = make16(EstadoG.IFR.valH,EstadoG.IFR.valL);
				
				/* Escreve os dados da distância do GP2D12 */
				printf( (unsigned char *)"GP2D12: high> ");
				PrintChar(EstadoG.IFR.valH);
				printf( (unsigned char *)" low> ");
				PrintChar(EstadoG.IFR.valL);
				
				utoa(valor_c,valor,base);
				
				printf( (unsigned char *)" , itoa: ");
				
				if(valor > 999)
				{
					n = 4;
				}	
				else if(valor > 99)
				{
					n = 3;
					UARTPut(' ');
					valor_c[3] = 0;
				}
				else if(valor > 9)
				{
					n = 2;
					UARTPut(' ');
					UARTPut(' ');
					valor_c[3] = 0;
					valor_c[2] = 0;
				}
				else
				{
					n = 1;
					UARTPut(' ');
					UARTPut(' ');
					UARTPut(' ');
					valor_c[3] = 0;
					valor_c[2] = 0;
					valor_c[1] = 0;
				}
				
				for(i=0;i<n;i++)
				{
					UARTPut(valor_c[i]);
					valor_c[i] = 0;
				}
				
				printf( (unsigned char *)" ;\r\n");
				
			}
			else
			{
				byte_check = Tipo;
				byte_check ^= EstadoG.IFR.valH;
				byte_check ^= EstadoG.IFR.valL;
				UARTPut(255);
				UARTPut(Tipo);
				UARTPut(EstadoG.IFR.valH);
				UARTPut(EstadoG.IFR.valL);
				UARTPut(byte_check);
			}		
			break;
		case LV_EZ0:
			Tipo = LV_EZ0_len;
			Tipo = Tipo << 4;
			Tipo += LV_EZ0;
			
			if(EstadoG.Modo_op == DEBUG)
			{
				unsigned int valor, i, n;
				unsigned char valor_c[4];
				int base=10;
				
				valor = make16(EstadoG.Sonar.valH,EstadoG.Sonar.valL);
				
				/* Escreve os dados da distância do GP2D12 */
				printf( (unsigned char *)"LV-EZ0: high> ");
				PrintChar(EstadoG.Sonar.valH);
				printf( (unsigned char *)" low> ");
				PrintChar(EstadoG.Sonar.valL);
				
				utoa(valor_c,valor,base);
				
				printf( (unsigned char *)" , itoa: ");
				
				if(valor > 999)
				{
					n = 4;
				}	
				else if(valor > 99)
				{
					n = 3;
					UARTPut(' ');
					valor_c[3] = 0;
				}
				else if(valor > 9)
				{
					n = 2;
					UARTPut(' ');
					UARTPut(' ');
					valor_c[3] = 0;
					valor_c[2] = 0;
				}
				else
				{
					n = 1;
					UARTPut(' ');
					UARTPut(' ');
					UARTPut(' ');
					valor_c[3] = 0;
					valor_c[2] = 0;
					valor_c[1] = 0;
				}
				
				for(i=0;i<n;i++)
				{
					UARTPut(valor_c[i]);
					valor_c[i] = 0;
				}
				
				printf( (unsigned char *)" ;\r\n");
			}
			else
			{
				byte_check = Tipo;
				byte_check ^= EstadoG.Sonar.valH;
				byte_check ^= EstadoG.Sonar.valL;
				UARTPut(255);
				UARTPut(Tipo);
				UARTPut(EstadoG.Sonar.valH);
				UARTPut(EstadoG.Sonar.valL);
				UARTPut(byte_check);
			}
			break;
	}	
	
}

/**
 * @fn Execute_Cmd(unsigned char *msg)
 * Executa o comando que foi recebido pela porta série.
 * @param *msg A mensagem recebida
 */
void Execute_Cmd(unsigned char *msg)
{
	switch(EstadoG.Received_type)
	{
		case 4: //Modo_op_cmd
			EstadoG.Received_cmd = msg[1];
			switch(EstadoG.Received_cmd)
			{
				case 1: //Modo_autonomo_req
					EstadoG.Modo_op = Autonomo;
					break;
				case 2: //Modo_manual_req
					EstadoG.Modo_op = Manual;
					break;
			}	
			break;
		case 5: //req_cmd
			EstadoG.Received_cmd = msg[1];
			switch(EstadoG.Received_cmd)
			{
				case 3: //mouse_req
					if(EstadoG.Modo_op == Manual)
					{
						EstadoG.Send.mouse = ON;
					}	
					break;
				case 4: //gp2d120_req
					if(EstadoG.Modo_op == Manual)
					{
						EstadoG.Send.gp2d12 = ON;
					}
					break;
				case 5: //lv_ez0_req
					if(EstadoG.Modo_op == Manual)
					{
						EstadoG.Send.lv_ez0 = ON;
					}
					break;
				default:
					break;
			}
			break;
		case 15: //Start_stop_cmd
			EstadoG.Received_cmd = msg[1];
			switch(EstadoG.Received_cmd)
			{
				case 85: //start_req
					EstadoG.Modo_op = Autonomo;
					break;
				case 170: //stop_req
					EstadoG.Modo_op = Idle;
					break;
				default:
					break;
			}
			break;
	}
}	

/**
 * @fn Process_Commands()
 * Rotina que verifica se existem mensagens para lêr no buffer da porta série.\n
 * Caso existam, verifica se são válidas e chama a rotina que as processa.
 */
void Process_Commands(void)
{
	BYTE c;
	
	int n;
	c = UART_Get();
	
	switch(c)
	{
		case 0xFF:
			{
				unsigned char type, msg_check,len, byte_test;
								
				*buf = (unsigned char)UART_Get();
					
				type = ((unsigned char)*buf) << 4;
				type = type >> 4;
				switch(type)
				{
					case 4:
						EstadoG.Received_type = op_mode_cmd;
						break;
					case 5:
						EstadoG.Received_type = req_cmd;
						break;
					case 15:
						EstadoG.Received_type = start_stop_cmd;
						break;
					default:
						EstadoG.Received_type = none;
						break;
				}
				if(EstadoG.Received_type != none)
				{
					msg_check = (unsigned char)*buf;
					len = ((unsigned char)*buf) >> 4;
										
					for(n=0;n<len;n++)
					{
						buf++;
						*buf = (unsigned char)UART_Get();
						msg_check ^= (unsigned char)*buf;
					}
					
					buf++;
					*buf = (unsigned char)UART_Get();
					byte_test = (unsigned char)*buf;
					
					if(byte_test == msg_check)
					{
						Execute_Cmd(str);
					}	
				}	
				
			}
			break;
		case 'm':
		case 'M':
			EstadoG.Modo_op = Manual;
			printf("\r\nEntrando no modo Manual...\r\nPara pedir dados precione a tecla r. \r\n");
			break;
		case 'r':
		case 'R':
			EstadoG.Send.mouse = ON;
			EstadoG.Send.gp2d12 = ON;
			EstadoG.Send.lv_ez0 = ON;
			printf("Pedido de dados enviado...\r\n");
			break;
		case 'd':
		case 'D':
			EstadoG.Modo_op = DEBUG;
			printf("\r\nEntrando no modo de Debug...\r\n");
			break;
		case 'a':
		case 'A':
			EstadoG.Modo_op = Autonomo;
			printf("Entrando no modo Autonomo...");
			break;
		case 'l':
		case 'L':
			LED6 = !LED6;
			break;
		default:
			break;
	}
	str[n] = 0;
	buf = &str[0];
}

/**
 * @fn is_get_ready()
 * Função que verifica se exitem bytes no buffer da porta série para ir ler.
 */
int is_get_ready(void)
{
	if((actualbuf != buffer))
	{
		return 1;
	}
	else
	{
		return 0;
	}	
}	

/**
 * @fn char UART_Get()
 * Função que lê o primeiro byte ainda não lido do buffer da porta série.
 * @return Devolve o primeiro byte não lido
 */
unsigned char UART_Get (void)
{
	unsigned char temp;
	
	temp = *actualbuf;
		
	if(actualbuf != lastbuf)
	{
		actualbuf++;
	}
	else
	{
		actualbuf = &msg[0];
	}
	
	return temp;
}	

/**
 * @fn interrupt isr()
 * Rotina de interrupts. Neste caso apenas vai tratar do interrupt
 * de recepção de um byte na porta série. E coloca esse byte num
 * buffer circular para que a porta série possa receber outros bytes
 * enquanto esse byte não for lido.
 */
void interrupt isr(void)
{
	GIE = 0;
	
	// Interrupt da porta série
	if(RCIF == 1)
	{
		*buffer = RCREG;
		CREN = 0;   // Disable UART receiver
        CREN = 1;   // Enable UART receiver
		if(buffer != lastbuf)
		{
			buffer++;
		}
		else
		{
			buffer = &msg[0];
		}
	}
	GIE = 1;
}